#ifndef _TUTAN_H
#define _TUTAN_H
/*
  Tutanchamun, an arcade game
  Copyright (C) 1998  David Kastrup
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
#include <setjmp.h>

extern jmp_buf gamebuf;
#define WIN 1
#define LOSE 2
#define GAME 3
#define QUIT 4

typedef unsigned char ubyte;
typedef unsigned short Tile[3][16];
typedef unsigned short Mask[16];
typedef struct {
  Tile tile;
  unsigned short mask[16];
} Sprite;

typedef struct {
  ubyte x,y;
} coor;

typedef struct _mcb {
  struct _mcb *next;
  ubyte flag;
  ubyte downcn;
  ubyte xcoor;
  ubyte ycoor;
  void **shape;
  struct _mph *mph;
} mcb;

#define PHASE 5
#define BIRTH 6
#define DEATH 7

extern mcb mcbarr[13];
#define mymcb mcbarr[0]
extern mcb * freemcb;
extern int nmons;

typedef struct _mph {
  ubyte inidow;
  ubyte distan;
  Sprite *shapsrc;
  ubyte monsco;
  void * shaplf[2];
  void * shapri[2];
} mph;

#define manmph mpharr[0]
#define manwmph mpharr[1]
#define snamph mpharr[2]
#define skumph mpharr[3]
#define spimph mpharr[4]
#define eggmph mpharr[5]
#define batmph mpharr[6]
#define whemph mpharr[7]
#define MPHLEN 8

extern mph mpharr[MPHLEN];

typedef struct {
  int timebyte;
  void (*timecode)(int *);
} timetab_t;

#define DEAD_TIMER -100
#define SHLBYT 0
#define SMLBYT 1
#define SHRBYT 2
#define SMRBYT 3
#define MOVBYT 4
#define TIMBYT 5
#define PROBYT 6

extern timetab_t timtab[];

void shotl(int *), shoml(int *), shotr(int *), shomr(int *),
  moveem(int *), timedo(int *), prod(int *);

void catchup(), startclock(), paus(int n);
int delay(unsigned n);

typedef struct {
  coor pos;		/* position of stove */
  ubyte downcn;		/* Down counter */
  ubyte propos;		/* production table position */
  unsigned long monpat;	/* activity pattern */
} protype;

typedef struct {
  ubyte width;			/* width of stage */
  ubyte xman, yman;
  ubyte y28, y92, y152, y220;	/* restart positions */
  int speed;
  ubyte colbak;			/* Background colour */
  ubyte colfor;			/* Brick colour */
  coor  addkey[2];		/* Positions of additional keys */
  coor  addhol[2];		/* Positions of additional keyholes */
  char time[4];			/* time to use in stage */
  mph  *mons[3];		/* Monster types */
  const ubyte (*stagesrc)[12];	/* Stage area source */
  protype  prodta[7];
} staget;

typedef struct {
  ubyte men;
  ubyte smarts;
  char score[7];
  ubyte wkey;
  ubyte nbonus;
  ubyte nstage;
  staget stage;
  ubyte staga[63][12];
} playfieldt;

extern playfieldt playfield[];
extern staget stages[16];
extern int player;
extern int scrx;
extern int fired;
void askfir();
void asksma();
void scrlft(), scrrig();
void add10(int), add100(int);
void qstove();
void killim(mcb *);
void killall();
void throwi(mcb *);
int demoplay();
void tutplay(int players);
int tutmain();
void chkhig();
void gethig();
void puthig();
void shohig();
void clrscr();
void playms(int player);
void bgoncesetup(), bgstagesetup(), bgstagedestroy();
void picsetup(), initshf();
extern void *d_pictur[4];

#define GA 0
#define ST 0x20
#define BO 0x40
#define BU 0x60
#define AO 0x80
#define AU 0xa0
#define SR 0xc0
#define LR 0xe0

#define XX 1
#define T9 3

#define EO 4
#define EU 5
#define ER 6
#define EL 7

#define OR 8
#define OL 9
#define UR 0xa
#define UL 0xb

#define OB 0xc
#define OX 0xd
#define UB 0xe
#define UX 0xf

#define SC 0x10
#define SX 0x11
#define LO 0x12

#define T1 0x18
#define T2 0x19
#define T3 0x1a
#define T4 0x1b
#define T5 0x1c
#define T6 0x1d
#define T7 0x1e
#define T8 0x1f

void draw(void *shape, int x, int y);
void undraw(void *shape, int x, int y);
void drawfull(void *shape, int x, int y);

extern int demflg;
extern int demcnt;
extern ubyte *dempnt;
int demo();
extern int colbak, colfor;

extern unsigned short pictur[4][3][48][3];

extern Sprite mansh[2], manwsh[2], snaksh[2], skulsh[2], spidsh[2],
              eggsh[2], batsh[2], wheesh[2], closh[2];
extern void *lampshd, *manshd, *scshd, *loshd, *ubshd, *obshd, *deadshd,
  *oxshd, *uxshd, *sxshd;

extern void **d_shbtab[32];
extern void *d_cloudshapes[2], *d_psh[6];

#endif
